function nntest_vf(fid, level,path, ax,bx, ay,by, at,bt, f)
% test the nonnegativity of f(x,y,t) on an interval [ax,bx]x[ay,by]x[at,bt] using interval arithmetics - vectorized version
% level, path ... auxiliary
% fid ... file identification number (must be opened)
% needs intlab package

% ----- update the statistics
global nntest_stat
if (nntest_stat{1} < level) 
  nntest_stat{1}=level;  % max level
end
nntest_stat{2} = nntest_stat{2} + 1;  % number of calls
% ----- end of update the statistics

N = 16;  % number of intervals in each variable 
         % The value 16 works fastest on my QuadCore processor.
N3 = N^3;


xd = [ax:(bx-ax)/N:bx]'; % column vector of length N+1
yd = [ay:(by-ay)/N:by]'; % column vector of length N+1
td = [at:(bt-at)/N:bt]'; % column vector of length N+1

x = infsup(xd(1:N),xd(2:N+1)); % column vector of N intervals
y = infsup(yd(1:N),yd(2:N+1)); % column vector of N intervals
t = infsup(td(1:N),td(2:N+1)); % column vector of N intervals

x = repmat(x,  [1,N,N]);   % 3 dimensional N x N x N array of copies of x;
y = repmat(y', [N,1,N]);   % 3 dimensional N x N x N array of copies of y;
t = repmat(reshape(t,1,1,N), [N,N,1]);  % 3 dimensional N x N x N array of copies of z;

x = reshape(x, N3,1);  % column vector of lenght N3 of copies of x;
y = reshape(y, N3,1);  % column vector of lenght N3 of copies of y;
t = reshape(t, N3,1);  % column vector of lenght N3 of copies of z;

rng = f(x,y,t);  % compute the range of of the function f

boolrng = inf(rng) < 0;  %  has value 1 in place of negative values; othervise the value is 0
res = sum( reshape(boolrng, N3,1) );  % number of "negative" subdomains
                                      % In "negative" subdomains the nonnegativity is not proved, yet.


% Output to the file fid 
nonnegidx = find( boolrng == 0);   % indices of the nonnegative subdomains
infrng = inf(rng);                 % infimum of rng (a vector of the left-endpoints of the intervals rng)
minval = min( infrng(nonnegidx) ); % the smallest nonnegative infimum of all intervals
fprintf(fid,'%d "%s" Testing [%g,%g]x[%g,%g]x[%g,%g]\n', level, path, ax,bx, ay,by, at,bt);
fprintf(fid,'%sNonnegative %d out of %d subdomains (N=%d) minimum=%g\n', blanks(5+length(path)), N3-res, N3,N, minval);
if (res > 0)
  fprintf(fid,'%sNonnegative subdomains:',blanks(5+length(path)));
  fprintf(fid,' %d', nonnegidx);
  fprintf(fid,'\n');
end

if (res > 0)
  % Output to the file fid
  fprintf(fid,'%sTesting remaining %d subdomains\n',blanks(5+length(path)),res);

  % Output to the screen
  fprintf('%d "%s" Testing [%g,%g]x[%g,%g]x[%g,%g] Negative %d out of %d (N=%d)\n', level, path, ax,bx, ay,by, at,bt, res, N3,N);

  % test (recursively) the nonnegativity of all "negative" subdomains
  for i = 1 : N3
    if (boolrng(i))
      nntest_vf(fid, level+1,[path ' ' num2str(i)], inf(x(i)),sup(x(i)), inf(y(i)),sup(y(i)), inf(t(i)),sup(t(i)), f);
    end % if
  end % for i

end % if
